import 'package:vitalapp/managers/interfaces/base.dart';

import 'models/device.dart';

abstract class IDeviceManager extends IManager {
  /// 获取授权的设备类型集合
  Future<List<String>> getAccessTypes();

  /// 获取授权的设备类型集合（本地缓存）
  Future<List<String>> getCacheAccessTypes();

  /// 获取设备列表
  Future<List<DeviceModel>> getDeviceList();

  /// 保存设备列表
  Future<void> saveDeviceList(List<DeviceModel> deviceList);

  /// 添加
  Future<void> addDevice(DeviceModel model);

  /// 删除设备
  Future<void> removeDeviceByType(String type);

  /// 上报设备(平板)
  Future<void> reportTablet();
}
